@@ -48,7 +48,7 @@ struct _HtmlBoxTextMaster {
 	guint must_relayout : 1;
 	guint preserved_leading_space : 1;
 	HtmlFontSpecification *font_spec;
-	HtmlWhiteSpaceType white_space:2;
+	guint white_space:2;
 };
 
 struct _HtmlBoxTextItemData {
@@ -89,10 +89,10 @@ html_box_text_destroy_slaves (HtmlBox *self)
 		if (html_box_text_is_master (HTML_BOX_TEXT (box)))
 			break;
 
-		box = box->next;
-
+		box = box->prev;
 		/* Now remove the box link */
 		html_box_remove (tmp_box);
+		box = box->next;
 		/* Free the memory */
 		g_object_unref (G_OBJECT (tmp_box));
 	}
@@ -149,11 +149,6 @@ html_box_text_finalize (GObject *self)
 		html_box_text_free_master (master);
 		g_free (master);
 		text->master = NULL;
-	} else {
-		box = HTML_BOX (self);
-
-		if (box->dom_node)
-			g_object_remove_weak_pointer (G_OBJECT (box->dom_node), (gpointer *) &(box->dom_node));
 	}
 
 	if (text->glyphs) {
@@ -1023,15 +1018,41 @@ html_box_text_should_paint (HtmlBox *box, GdkRectangle *area, gint tx, gint ty)
 	return TRUE;
 }
 
+static AtkObject*
+html_box_text_get_accessible (HtmlBoxText *text)
+{
+	AtkObject *obj;
+
+	if (html_box_text_get_len (text) == 0)
+		return NULL;
+
+	obj = atk_gobject_accessible_for_object (G_OBJECT (text));
+	/* Accessibility is not enabled */
+	if (ATK_IS_NO_OP_OBJECT (obj))
+		return NULL;
+	return obj;
+}
+
 void 
 html_box_text_set_selection (HtmlBoxText *text, HtmlBoxTextSelection mode, gint start_index, gint end_index)
 {
+	AtkObject *obj;
+
+	if (text->selection == mode &&
+	    text->sel_start_index == start_index &&
+            text->sel_end_index == end_index)
+		return;
+ 
 	text->selection = mode;
 
 	if (start_index >= 0)
 		text->sel_start_index = start_index;
 	if (end_index >= 0)
 		text->sel_end_index = end_index;
+
+	obj = html_box_text_get_accessible (text);
+	if (obj)
+		g_signal_emit_by_name (obj, "text-selection-changed");
 }
 
 static void
